package com.brianhadaway
{
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	import flash.geom.Matrix;
	import flash.display.Sprite;
	import flash.display.BitmapData;
	import flash.display.Bitmap;
	import flash.text.TextField;
	import flash.media.Camera;
	import flash.media.Video;
	import flash.geom.Point;
	import flash.geom.Rectangle;

	/**
	 * 
	 * BasicTracker
	 * 
	 * @version 1.0 | 4/27/2010
	 * @author Brian Hadaway
	 * 
	 * The BasicTracker is a class used for motion tracking with
	 * a webcam in ActionScript 3
	 * 
	 * Visit http://www.brianhadway.com/ for more info
	 * 
	 */
	 
	public class BasicTracker extends Sprite 
	{
		
		// ----------------------------------
		// PRIVATE MEMBERS
		// ----------------------------------
		
		
		private var _camera : Camera;
		private var _video : Video;
		private var _output : BitmapData;
		private var _difference : BitmapData;
		private var _previous : BitmapData;
		private var _label : TextField;
		private var _renderTimer : Timer;
		private var _cameraFPS : Number = 30;
		private var _cameraW : Number = 550;
		private var _cameraH : Number = 400;
		private var _bm : Bitmap;
		private var _matrix : Matrix;
		private var _showVideo : Boolean = true;
		private var _renderFrequency : Number = 100;
		private var _thresholdMask : uint = 0xFF333333;
		private var _showMotion : Boolean = true;
		private var _motionArea : Rectangle = new Rectangle();
		private var _motionColor : uint = 0xFF00FF00;

		// ----------------------------------
		// CONSTRUCTOR
		// ----------------------------------
		
		public function BasicTracker() 
		{							
			init();
		}
		
		// ----------------------------------
		// PUBLIC METHODS
		// ----------------------------------
		
		/**
		 * Initialize 
		 */
		
		public function init() : void
		{
			_camera = Camera.getCamera();
			_camera.setMode(_cameraW, _cameraH, _cameraFPS);
			if (_camera == null) {
				_label = new TextField();
				_label.text = "NO WEBCAM";
				addChild(_label);
			} 
			else 
			{
				_matrix = new Matrix(-1,0,0,1,_camera.width,0);
				_video = new Video(_camera.width, _camera.height);
				_video.transform.matrix = _matrix;
				_video.attachCamera(_camera);
				addChild(_video);
				_video.visible = false;

				_output = new BitmapData(_video.width, _video.height, true, 0x00000000);
				_difference = new BitmapData(_video.width, _video.height);
				_previous = new BitmapData(_video.width, _video.height);
				
				_bm = new Bitmap(_output);
				addChild(_bm);
				_bm.visible = _showMotion;
								
				_renderTimer = new Timer(_renderFrequency);
				_renderTimer.addEventListener(TimerEvent.TIMER, render, false, 0, true);
				_renderTimer.start();
				
			}
		}
		
		/**
		 * Render tracked motion
		 * 
		 * @param e  TimerEvent dispatched by _renderTimer
		 */

		public function render(e:TimerEvent = null) : void 
		{
			if (!_camera.currentFPS) return;
			
			_video.visible = _showVideo;
			_difference.draw(_video,_matrix);
			_difference.draw(_previous,_matrix,null,"difference");
			_output.fillRect(new Rectangle(0,0,output.width,output.height),0x00000000);
			_output.threshold(_difference, new Rectangle(0,0,_difference.width,_difference.height), new Point(0,0), ">", _thresholdMask, _motionColor);
			_previous.draw(_video);
			_motionArea = output.getColorBoundsRect(0xFFFFFFFF, _motionColor);
		}
		
		// ----------------------------------
		// PUBLIC ACCESSORS 
		// ----------------------------------
		
		/**
		 * BitmapData used to track motion
		 */
		public function get output() : BitmapData 
		{
			return _output;
		}
		
		/**
		 * Whether video is displayed
		 */
		public function get showVideo() : Boolean 
		{
			return _showVideo;
		}
		
		public function set showMotion(value : Boolean) : void 
		{
			_showMotion = value;
		}
		
		/**
		 * Whether motion is displayed
		 */
		public function get showMotion() : Boolean 
		{
			return _showMotion;
		}
		
		public function set showVideo(value : Boolean) : void 
		{
			_showVideo = value;
		}
		
		/**
		 * Camera width
		 */
		public function get cameraW() : uint 
		{
			return _cameraW;
		}
		
		public function set cameraW(value : uint) : void 
		{
			_cameraW = value;
		}
		
		/**
		 * Camera height
		 */
		public function get cameraH() : uint 
		{
			return _cameraH;
		}
		
		public function set cameraH(value : uint) : void 
		{
			_cameraH = value;	
		}
		
		/**
		 * Camera FPS
		 */
		public function get cameraFPS() : uint 
		{
			return _cameraFPS;
		}
		
		public function set cameraFPS(value : uint) : void 
		{
			_cameraFPS = value;
		}
		
		/**
		 * Frequency that motion is tracked (milliseconds)
		 */		
		public function get renderFrequency() : uint 
		{
			return _renderFrequency;
		}
		
		public function set renderFrequency(value : uint) : void 
		{
			_renderFrequency = value;
		}
		
		/**
		 * Mask color used to isolate motion
		 */
		public function get thresholdMask() : uint 
		{
			return _thresholdMask;
		}
		
		public function set thresholdMask(value : uint) : void 
		{
			_thresholdMask = value;
		}
		
		/**
		 * Rectangle that bounds motionArea
		 */
		public function get motionArea() : Rectangle 
		{
			return _motionArea;
		}
		
		/**
		 * Color used in rendering motion visualization (ARGB)
		 */
		public function get motionColor() : uint 
		{
			return _motionColor;
		}
		
		public function set motionColor(value : uint) : void 
		{
			_motionColor = value;
		}
	}
}